@@ -13,27 +13,106 @@ from .. import units as plonk_units
 from ..snap import SnapLike
 from ..utils.math import cross, norm
 
-_units = {
-    'momentum': 'momentum',
+array_units = {
     'angular_momentum': 'angular_momentum',
-    'specific_angular_momentum': 'specific_angular_momentum',
-    'kinetic_energy': 'energy',
-    'specific_kinetic_energy': 'specific_energy',
-    'semi_major_axis': 'length',
-    'eccentricity': 'dimensionless',
-    'inclination': 'radian',
-    'gas_mass': 'mass',
-    'dust_mass': 'mass',
-    'gas_density': 'density',
-    'dust_density': 'density',
-    'radial_distance': 'length',
-    'azimuthal_angle': 'radian',
-    'polar_angle': 'radian',
-    'radial_velocity': 'velocity',
     'angular_velocity': 'velocity',
+    'azimuthal_angle': 'radian',
+    'dust_density': 'density',
+    'dust_fraction': 'dimensionless',
+    'dust_mass': 'mass',
+    'eccentricity': 'dimensionless',
+    'gas_density': 'density',
+    'gas_fraction': 'dimensionless',
+    'gas_mass': 'mass',
+    'inclination': 'radian',
+    'keplerian_frequency': 'frequency',
+    'kinetic_energy': 'energy',
+    'momentum': 'momentum',
+    'polar_angle': 'radian',
+    'radial_distance': 'length',
+    'radial_velocity': 'velocity',
+    'semi_major_axis': 'length',
+    'specific_angular_momentum': 'specific_angular_momentum',
+    'specific_kinetic_energy': 'specific_energy',
+    'stokes_number': 'dimensionless',
     'temperature': 'temperature',
 }
 
+array_requires = {
+    'angular_momentum': ('mass', 'position', 'velocity'),
+    'angular_velocity': ('position', 'velocity'),
+    'azimuthal_angle': ('position',),
+    'dust_density': ('dust_fraction',),
+    'dust_fraction': ('dust_fraction',),
+    'dust_mass': ('dust_fraction',),
+    'eccentricity': ('position', 'velocity'),
+    'gas_density': ('dust_fraction',),
+    'gas_fraction': ('dust_fraction',),
+    'gas_mass': ('dust_fraction',),
+    'inclination': ('mass', 'position', 'velocity'),
+    'keplerian_frequency': ('position',),
+    'kinetic_energy': ('mass', 'velocity'),
+    'momentum': ('mass', 'velocity'),
+    'polar_angle': ('position',),
+    'radial_distance': ('position',),
+    'radial_velocity': ('position', 'velocity'),
+    'semi_major_axis': ('position', 'velocity'),
+    'specific_angular_momentum': ('position', 'velocity'),
+    'specific_kinetic_energy': 'velocity',
+    'stokes_number': ('position', 'stopping_time'),
+    'temperature': ('sound_speed',),
+}
+
+array_rotatable = {
+    'angular_momentum': True,
+    'angular_velocity': False,
+    'azimuthal_angle': False,
+    'dust_density': False,
+    'dust_fraction': False,
+    'dust_mass': False,
+    'eccentricity': False,
+    'gas_density': False,
+    'gas_fraction': False,
+    'gas_mass': False,
+    'inclination': False,
+    'keplerian_frequency': False,
+    'kinetic_energy': False,
+    'momentum': True,
+    'polar_angle': False,
+    'radial_distance': False,
+    'radial_velocity': False,
+    'semi_major_axis': False,
+    'specific_angular_momentum': True,
+    'specific_kinetic_energy': False,
+    'stokes_number': False,
+    'temperature': False,
+}
+
+array_dust = {
+    'angular_momentum': False,
+    'angular_velocity': False,
+    'azimuthal_angle': False,
+    'dust_density': True,
+    'dust_fraction': True,
+    'dust_mass': True,
+    'eccentricity': False,
+    'gas_density': False,
+    'gas_fraction': False,
+    'gas_mass': False,
+    'inclination': False,
+    'keplerian_frequency': False,
+    'kinetic_energy': False,
+    'momentum': False,
+    'polar_angle': False,
+    'radial_distance': False,
+    'radial_velocity': False,
+    'semi_major_axis': False,
+    'specific_angular_momentum': False,
+    'specific_kinetic_energy': False,
+    'stokes_number': True,
+    'temperature': False,
+}
+
 
 def momentum(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     """Calculate the momentum.
@@ -324,22 +403,18 @@ def semi_major_axis(
 
     radius = norm(pos, axis=1)
 
-    specific_angular_momentum = cross(pos, vel)
-    specific_angular_momentum_magnitude = norm(specific_angular_momentum, axis=1)
+    _specific_angular_momentum = cross(pos, vel)
+    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
 
-    specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
+    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
     specific_potential_energy = -mu / radius
-    specific_energy = specific_kinetic_energy + specific_potential_energy
+    specific_energy = _specific_kinetic_energy + specific_potential_energy
 
     term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
 
-    eccentricity = np.sqrt(1 + 2 * term)
+    _eccentricity = np.sqrt(1 + 2 * term)
 
-    semi_major_axis = specific_angular_momentum_magnitude ** 2 / (
-        mu * (1 - eccentricity ** 2)
-    )
-
-    return semi_major_axis
+    return specific_angular_momentum_magnitude ** 2 / (mu * (1 - _eccentricity ** 2))
 
 
 def eccentricity(
@@ -388,21 +463,19 @@ def eccentricity(
 
     radius = norm(pos, axis=1)
 
-    specific_angular_momentum = cross(pos, vel)
-    specific_angular_momentum_magnitude = norm(specific_angular_momentum, axis=1)
+    _specific_angular_momentum = cross(pos, vel)
+    specific_angular_momentum_magnitude = norm(_specific_angular_momentum, axis=1)
 
-    specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
+    _specific_kinetic_energy = 1 / 2 * norm(vel, axis=1) ** 2
     specific_potential_energy = -mu / radius
-    specific_energy = specific_kinetic_energy + specific_potential_energy
+    specific_energy = _specific_kinetic_energy + specific_potential_energy
 
     term = specific_energy * (specific_angular_momentum_magnitude / mu) ** 2
-    eccentricity = np.sqrt(1 + 2 * term)
-
-    return eccentricity
+    return np.sqrt(1 + 2 * term)
 
 
 def inclination(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
-    """Calculate the inclination with respect to the xy-plane.
+    """Calculate the inclination.
 
     The inclination is calculated by taking the angle between the
     angular momentum vector and the z-axis, with the angular momentum
@@ -433,14 +506,12 @@ def inclination(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     origin = (mass[:, np.newaxis] * pos).sum(axis=0) / mass.sum()
     pos = pos - origin
 
-    specific_angular_momentum = cross(pos, vel)
+    _specific_angular_momentum = cross(pos, vel)
 
-    inclination = np.arccos(
-        specific_angular_momentum[:, 2] / norm(specific_angular_momentum, axis=1)
+    return np.arccos(
+        _specific_angular_momentum[:, 2] / norm(_specific_angular_momentum, axis=1)
     )
 
-    return inclination
-
 
 def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     """Calculate the gas fraction.
@@ -460,21 +531,21 @@ def gas_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The gas fraction on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
-        gas_fraction = 1 - dust_fraction.sum(axis=1)
+        _dust_fraction: ndarray = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         particle_type = snap['type']
-        gas_fraction = np.ones(len(snap))
-        gas_fraction[particle_type != snap.particle_type['gas']] = 0
+        _gas_fraction = np.ones(len(snap))
+        _gas_fraction[particle_type != snap.particle_type['gas']] = 0
 
     else:
-        gas_fraction = np.ones(len(snap))
+        _gas_fraction = np.ones(len(snap))
 
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        return gas_fraction[h > 0]
-    return gas_fraction
+        return _gas_fraction[h > 0]
+    return _gas_fraction
 
 
 def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
@@ -495,19 +566,19 @@ def gas_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The gas mass on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
-        gas_fraction = 1 - dust_fraction.sum(axis=1)
+        _dust_fraction: ndarray = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         particle_type = snap['type']
-        gas_fraction = np.zeros(len(snap))
-        gas_fraction[particle_type == snap.particle_type['gas']] = 1
+        _gas_fraction = np.zeros(len(snap))
+        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
 
     mass: ndarray = snap['mass']
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        return (mass * gas_fraction)[h > 0]
-    return mass * gas_fraction
+        return (mass * _gas_fraction)[h > 0]
+    return mass * _gas_fraction
 
 
 def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
@@ -528,19 +599,19 @@ def gas_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The gas density on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
-        gas_fraction = 1 - dust_fraction.sum(axis=1)
+        _dust_fraction: ndarray = snap['dust_fraction']
+        _gas_fraction = 1 - _dust_fraction.sum(axis=1)
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         particle_type = snap['type']
-        gas_fraction = np.zeros(len(snap))
-        gas_fraction[particle_type == snap.particle_type['gas']] = 1
+        _gas_fraction = np.zeros(len(snap))
+        _gas_fraction[particle_type == snap.particle_type['gas']] = 1
 
     density: ndarray = snap['density']
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        return (density * gas_fraction)[h > 0]
-    return density * gas_fraction
+        return (density * _gas_fraction)[h > 0]
+    return density * _gas_fraction
 
 
 def dust_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
@@ -561,22 +632,23 @@ def dust_fraction(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The gas fraction on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
+        _dust_fraction: ndarray = snap['dust_fraction']
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         n_dust = len(snap.properties.get('grain_size', []))
-        dust_type = snap['dust_type']
-        dust_fraction = np.zeros((len(snap), n_dust))
+        sub_type = snap['sub_type']
+        type_mask = snap['type'] == snap.particle_type['dust']
+        _dust_fraction = np.zeros((len(snap), n_dust))
         for idx in range(1, n_dust + 1):
-            dust_fraction[dust_type == idx, idx - 1] = 1
+            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
 
     else:
         raise ValueError('No dust available')
 
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        return dust_fraction[h > 0]
-    return dust_fraction
+        return _dust_fraction[h > 0]
+    return _dust_fraction
 
 
 def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
@@ -597,14 +669,15 @@ def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The dust mass per species on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
+        _dust_fraction: ndarray = snap['dust_fraction']
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         n_dust = len(snap.properties.get('grain_size', []))
-        dust_type = snap['dust_type']
-        dust_fraction = np.zeros((len(snap), n_dust))
+        sub_type = snap['sub_type']
+        type_mask = snap['type'] == snap.particle_type['dust']
+        _dust_fraction = np.zeros((len(snap), n_dust))
         for idx in range(1, n_dust + 1):
-            dust_fraction[dust_type == idx, idx - 1] = 1
+            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
 
     else:
         raise ValueError('No dust available')
@@ -612,9 +685,9 @@ def dust_mass(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     mass: ndarray = snap['mass']
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        dust_mass = mass[:, np.newaxis] * dust_fraction
-        return dust_mass[h > 0]
-    return mass[:, np.newaxis] * dust_fraction
+        _dust_mass = mass[:, np.newaxis] * _dust_fraction
+        return _dust_mass[h > 0]
+    return mass[:, np.newaxis] * _dust_fraction
 
 
 def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
@@ -635,14 +708,15 @@ def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
         The dust density per species on the particles.
     """
     if snap.properties['dust_method'] == 'dust/gas mixture':
-        dust_fraction: ndarray = snap['dust_fraction']
+        _dust_fraction: ndarray = snap['dust_fraction']
 
     elif snap.properties['dust_method'] == 'dust as separate sets of particles':
         n_dust = len(snap.properties.get('grain_size', []))
-        dust_type = snap['dust_type']
-        dust_fraction = np.zeros((len(snap), n_dust))
+        sub_type = snap['sub_type']
+        type_mask = snap['type'] == snap.particle_type['dust']
+        _dust_fraction = np.zeros((len(snap), n_dust))
         for idx in range(1, n_dust + 1):
-            dust_fraction[dust_type == idx, idx - 1] = 1
+            _dust_fraction[type_mask & (sub_type == idx), idx] = 1
 
     else:
         raise ValueError('No dust available')
@@ -650,9 +724,9 @@ def dust_density(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
     density: ndarray = snap['density']
     if ignore_accreted:
         h: ndarray = snap['smoothing_length']
-        dust_mass = density[:, np.newaxis] * dust_fraction
-        return dust_mass[h > 0]
-    return density[:, np.newaxis] * dust_fraction
+        _dust_mass = density[:, np.newaxis] * _dust_fraction
+        return _dust_mass[h > 0]
+    return density[:, np.newaxis] * _dust_fraction
 
 
 def radial_distance(
@@ -687,10 +761,9 @@ def radial_distance(
 
     if coordinates == 'cylindrical':
         return np.sqrt(x ** 2 + y ** 2)
-    elif coordinates == 'spherical':
+    if coordinates == 'spherical':
         return np.sqrt(x ** 2 + y ** 2 + z ** 2)
-    else:
-        raise ValueError('Cannot determine coordinates')
+    raise ValueError('Cannot determine coordinates')
 
 
 def azimuthal_angle(snap: SnapLike, ignore_accreted: bool = False) -> ndarray:
