@@ -13,7 +13,7 @@ import pandas as pd
 from numpy import ndarray
 from pandas import DataFrame
 
-from .. import Quantity
+from .. import Quantity, logger
 from .. import units as plonk_units
 from ..snap import SnapLike, gravitational_constant_in_code_units
 from ..utils import average, is_documented_by
@@ -36,7 +36,8 @@ class Profile:
     ndim : optional
         The dimension of the profile. For ndim == 2, the radial binning
         is cylindrical in the xy-plane. For ndim == 3, the radial
-        binning is spherical. Default is 2.
+        binning is spherical. For ndim == 1, the radial binning is
+        Cartesian along the x-axis. Default is 2.
     radius_min : optional
         The minimum radius for binning. Defaults to minimum on the
         particles.
@@ -137,8 +138,8 @@ class Profile:
 
         self.snap = snap
         self.ndim = ndim
-        self.aggregation = self._check_aggregation(aggregation)
-        self.spacing = self._check_spacing(spacing)
+        self.aggregation = _check_aggregation(aggregation)
+        self.spacing = _check_spacing(spacing)
         self.properties: Dict[str, Any] = {}
 
         self._profiles: Dict[str, ndarray] = {}
@@ -170,20 +171,6 @@ class Profile:
         n_dust = len(self.snap.properties.get('grain_size', []))
         _generate_profiles(n_dust)
 
-    def _check_aggregation(self, method: str) -> str:
-        if method in _aggregations:
-            return method
-        raise ValueError(
-            f'Cannot determine aggregation method: choose from {_aggregations}'
-        )
-
-    def _check_spacing(self, spacing: str) -> str:
-        if spacing.lower() in ('lin', 'linear'):
-            return 'linear'
-        elif spacing.lower() in ('log', 'logarithm', 'logarithmic'):
-            return 'log'
-        raise ValueError('Cannot determine spacing')
-
     def _setup_particle_mask(self, ignore_accreted: bool) -> ndarray:
         if ignore_accreted is False:
             return np.ones(len(self.snap), dtype=bool)
@@ -193,18 +180,20 @@ class Profile:
     def _calculate_x(self) -> ndarray:
         pos: ndarray = self.snap['xyz']
         pos = pos[self._mask]
+        if self.ndim == 1:
+            return pos[:, 0]
         if self.ndim == 2:
             return np.sqrt(pos[:, 0] ** 2 + pos[:, 1] ** 2)
-        elif self.ndim == 3:
+        if self.ndim == 3:
             return np.sqrt(pos[:, 0] ** 2 + pos[:, 1] ** 2 + pos[:, 2] ** 2)
+        raise ValueError('Unknown ndim: cannot calculate x array')
 
     def _set_range(
         self, radius_min: Optional[Any], radius_max: Optional[Any]
     ) -> Tuple[float, float]:
         if self.snap._physical_units:
             return self._set_range_physical_units(radius_min, radius_max)
-        else:
-            return self._set_range_code_units(radius_min, radius_max)
+        return self._set_range_code_units(radius_min, radius_max)
 
     def _set_range_code_units(
         self, radius_min: Optional[float], radius_max: Optional[float]
@@ -249,7 +238,9 @@ class Profile:
             bin_edges = self._bin_edges_physical_units()
         else:
             bin_edges = self._bin_edges_code_units()
-        if self.ndim == 2:
+        if self.ndim == 1:
+            bin_sizes = bin_edges[1:] - bin_edges[:-1]
+        elif self.ndim == 2:
             bin_sizes = np.pi * (bin_edges[1:] ** 2 - bin_edges[:-1] ** 2)
         elif self.ndim == 3:
             bin_sizes = 4 / 3 * np.pi * (bin_edges[1:] ** 3 - bin_edges[:-1] ** 3)
@@ -305,7 +296,7 @@ class Profile:
 
         if name in self._profiles:
             return self._profiles[name]
-        elif name in Profile._profile_functions:
+        if name in Profile._profile_functions:
             self._profiles[name] = Profile._profile_functions[name](self)
             return self._profiles[name]
 
@@ -317,18 +308,11 @@ class Profile:
             array_name = name
         try:
             array: ndarray = self.snap[array_name]
-        except ValueError:
-            if self.snap._extra_quantities:
-                print('Profile unavailable.')
-            else:
-                print(
-                    'Profile unavailable. Try calling extra_quantities method on Snap.'
+            if array.ndim == 1:
+                self._profiles[name] = self.particles_to_binned_quantity(
+                    aggregation, array
                 )
-            return
-        if array.ndim == 1:
-            self._profiles[name] = self.particles_to_binned_quantity(aggregation, array)
-            return self._profiles[name]
-        else:
+                return self._profiles[name]
             raise ValueError(
                 'Requested profile has array dimension > 1.\nTo access x-, y-, or '
                 'z-components, or magnitude of vector quantities,\ntry, for '
@@ -336,6 +320,13 @@ class Profile:
                 'access dust profiles, try, for example, prof["stopping_time_001"] '
                 'or\nprof["dust_mass_sum"].'
             )
+        except ValueError:
+            if self.snap._extra_quantities:
+                logger.error('Profile unavailable.')
+            else:
+                logger.error(
+                    'Profile unavailable. Try calling extra_quantities method on Snap.'
+                )
 
     def __getitem__(self, name: str) -> ndarray:
         """Return the profile of a given kind."""
@@ -352,7 +343,7 @@ class Profile:
                 'Attempting to overwrite existing profile. To do so, first delete the '
                 'profile\nwith del prof["profile"], then try again.'
             )
-        elif name in self.available_profiles():
+        if name in self.available_profiles():
             raise ValueError(
                 'Attempting to set profile already available. '
                 'See prof.available_profiles().'
@@ -375,6 +366,10 @@ class Profile:
         """Dunder str method."""
         return f'<plonk.Profile "{self.snap.file_path.name}">'
 
+    def _ipython_key_completions_(self):
+        """Tab completion for IPython __getitem__ method."""
+        return self.available_profiles()
+
     def loaded_profiles(self):
         """Return a listing of loaded profiles."""
         return tuple(sorted(self._profiles.keys()))
@@ -417,6 +412,11 @@ class Profile:
             A matplotlib Axes object to plot to.
         **kwargs
             Keyword arguments to pass to Axes plot method.
+
+        Returns
+        -------
+        ax
+            The matplotlib Axes object.
         """
         if isinstance(y, str):
             y = [y]
@@ -435,13 +435,10 @@ class Profile:
         if x_unit is not None:
             if not self.snap._physical_units:
                 raise ValueError('Cannot set unit if snap is not in physical units')
-            else:
-                _x = _x.to(x_unit)
+            _x = _x.to(x_unit)
 
         if ax is None:
-            fig, ax = plt.subplots()
-        else:
-            fig = ax.get_figure()
+            _, ax = plt.subplots()
 
         xlabel = x.capitalize().replace('_', ' ')
         if self.snap._physical_units:
@@ -473,7 +470,10 @@ class Profile:
                 if std_dev_shading:
                     _y_std = _y_std.magnitude
                     _y_mean = _y_mean.magnitude
-            ax.plot(_x, _y, label=label, **kwargs)
+            if kwargs.get('label') is None:
+                ax.plot(_x, _y, label=label, **kwargs)
+            else:
+                ax.plot(_x, _y, **kwargs)
             if std_dev_shading:
                 color = ax.lines[idx].get_color()
                 ax.fill_between(
@@ -482,7 +482,7 @@ class Profile:
 
         ax.legend()
 
-        return fig, ax
+        return ax
 
     def to_dataframe(
         self, columns: Union[Tuple[str, ...], List[str]] = None
@@ -658,6 +658,7 @@ def load_profile(
     spacing: str = 'linear',
     ignore_accreted: bool = True,
 ) -> Profile:
+    logger.debug(f'Loading profile: {snap.file_path.name}')
     return Profile(
         snap=snap,
         ndim=ndim,
@@ -668,3 +669,19 @@ def load_profile(
         spacing=spacing,
         ignore_accreted=ignore_accreted,
     )
+
+
+def _check_aggregation(method: str) -> str:
+    if method in _aggregations:
+        return method
+    raise ValueError(
+        f'Cannot determine aggregation method: choose from {_aggregations}'
+    )
+
+
+def _check_spacing(spacing: str) -> str:
+    if spacing.lower() in ('lin', 'linear'):
+        return 'linear'
+    if spacing.lower() in ('log', 'logarithm', 'logarithmic'):
+        return 'log'
+    raise ValueError('Cannot determine spacing')
