@@ -10,10 +10,13 @@ import h5py
 import numpy as np
 from numpy import ndarray
 
+from ... import logger
 from ... import units as plonk_units
 from ..snap import Snap
 from ..units import generate_units_dictionary
 
+igas, iboundary, istar, idarkmatter, ibulge = 1, 3, 4, 5, 6
+maxdusttypes = 100
 _bignumber = 1e29
 
 _particle_array_name_map = {
@@ -71,6 +74,7 @@ def generate_snap_from_file(filename: Union[str, Path]) -> Snap:
     Snap
         A Snap object.
     """
+    logger.debug(f'Loading Phantom snapshot: {filename}')
     file_path = pathlib.Path(filename).expanduser()
     if not file_path.is_file():
         raise FileNotFoundError('Cannot find snapshot file')
@@ -173,11 +177,14 @@ def _populate_particle_array_registry(
         arrays.remove('dustfrac')
 
     elif ndustlarge > 0:
-        # Read dust type if there are dust particles
-        array_registry['dust_type'] = _dust_particle_type
-        array_registry['stopping_time'] = _stopping_time
         array_registry['dust_to_gas_ratio'] = _dust_to_gas_ratio
         arrays.remove('dustfrac')
+        # Currently only dust as species has particle sub-types
+        array_registry['sub_type'] = _sub_type
+
+    if ndustsmall > 0 or ndustlarge > 0 and 'tstop' in arrays:
+        array_registry['stopping_time'] = _stopping_time
+        arrays.remove('tstop')
 
     # Read arrays if available
     for name_on_file, name in name_map.items():
@@ -216,25 +223,67 @@ def _get_dataset(dataset: str, group: str) -> Callable:
 
 
 def _particle_type(snap: Snap) -> ndarray:
+    # Type        | Phantom                               | Plonk
+    # ----------- | ------------------------------------- | -----
+    # Gas         |                                     1 | 1
+    # Dust        |      idust -> idust + ndustlarge      | 2
+    # Boundary    | idustbound -> idustbound + ndustlarge | 3
+    # Star        |                                     4 | 4
+    # Dark matter |                                     5 | 5
+    # Bulge       |                                     6 | 6
     idust = _get_dataset('idust', 'header')(snap)
+    ndustlarge = _get_dataset('ndustlarge', 'header')(snap)
     particle_type = np.abs(_get_dataset('itype', 'particles')(snap))
-    particle_type[particle_type >= idust] = 2
+    particle_type[
+        (particle_type >= idust) & (particle_type < idust + ndustlarge)
+    ] = snap.particle_type['dust']
+    try:
+        idustbound = _get_dataset('idustbound', 'header')(snap)
+        particle_type[
+            (particle_type >= idustbound) & (particle_type < idustbound + ndustlarge)
+        ] = snap.particle_type['boundary']
+    except KeyError:
+        if np.any(particle_type >= idust + ndustlarge):
+            logger.error('Cannot determine dust boundary particles')
     return particle_type
 
 
-def _dust_particle_type(snap: Snap) -> ndarray:
-    idust = _get_dataset('idust', 'header')(snap)
+def _sub_type(snap: Snap) -> ndarray:
+    #           | Types
+    #           | Gas | Dust | Boundary
+    # Sub-types |     |      |
+    # --------- | --- | ---- | --------
+    # Gas       |  0  |  n/a |  0
+    # Dust 1    | n/a |   1  |  1
+    # Dust 2    | n/a |   2  |  2
+    # Dust 3    | n/a |   3  |  3
+    # ...
     particle_type = np.abs(_get_dataset('itype', 'particles')(snap))
-    dust_type = np.zeros(particle_type.shape, dtype=np.int8)
-    dust_type[particle_type >= idust] = (
-        particle_type[particle_type >= idust] - idust + 1
-    )
-    return dust_type
+    sub_type = np.zeros(particle_type.shape, dtype=np.int8)
+    sub_type[
+        (particle_type == igas)
+        | (particle_type == istar)
+        | (particle_type == idarkmatter)
+        | (particle_type == ibulge)
+    ] = 0
+    sub_type[particle_type == iboundary] = 0
+    idust = _get_dataset('idust', 'header')(snap)
+    ndustlarge = _get_dataset('ndustlarge', 'header')(snap)
+    for idx in range(idust, idust + ndustlarge):
+        sub_type[particle_type == idx] = idx - idust + 1
+    try:
+        idustbound = _get_dataset('idustbound', 'header')(snap)
+        for idx in range(idustbound, idustbound + ndustlarge):
+            sub_type[particle_type == idx] = idx - idustbound + 1
+    except KeyError:
+        if np.any(particle_type >= idust + ndustlarge):
+            logger.error('Cannot determine dust boundary particles')
+    return sub_type
 
 
 def _mass(snap: Snap) -> ndarray:
-    massoftype = snap._file_pointer['header/massoftype'][()]
-    particle_type = _get_dataset('itype', 'particles')(snap)
+    massoftype = _get_dataset('massoftype', 'header')(snap)
+    particle_type = np.abs(_get_dataset('itype', 'particles')(snap))
     return massoftype[particle_type - 1]
 
 
@@ -242,47 +291,40 @@ def _density(snap: Snap) -> ndarray:
     m = _mass(snap)
     h = _get_dataset('h', 'particles')(snap)
     hfact = snap.properties['smoothing_length_factor']
-    rho = np.zeros(m.shape)
-    h = np.abs(h)
-    rho[h > 0] = m[h > 0] * (hfact / h[h > 0]) ** 3
-    return rho
+    return m * (hfact / np.abs(h)) ** 3
 
 
 def _pressure(snap: Snap) -> ndarray:
-    ieos = snap._file_pointer['header/ieos'][()]
+    ieos = _get_dataset('ieos', 'header')(snap)
     K = snap.properties['polytropic_constant']
     gamma = snap.properties['adiabatic_index']
     rho = _density(snap)
     if ieos == 1:
         return K * rho
-    elif ieos == 2:
+    if ieos == 2:
         return K * rho ** (gamma - 1)
-    elif ieos == 3:
+    if ieos == 3:
         q = snap.properties['sound_speed_index']
-        pos: ndarray = _get_dataset('xyz', 'particles')(snap)
+        pos = _get_dataset('xyz', 'particles')(snap)
         r_squared = pos[:, 0] ** 2 + pos[:, 1] ** 2 + pos[:, 2] ** 2
         return K * rho * r_squared ** (-q)
-    else:
-        raise ValueError('Cannot determine equation of state')
+    raise ValueError('Unknown equation of state')
 
 
 def _sound_speed(snap: Snap) -> ndarray:
-    ieos = snap._file_pointer['header/ieos'][()]
+    ieos = _get_dataset('ieos', 'header')(snap)
     gamma = snap.properties['adiabatic_index']
     rho = _density(snap)
     P = _pressure(snap)
-    cs = np.zeros(P.shape)
     if ieos in (1, 3):
-        cs[rho > 0] = np.sqrt(P[rho > 0] / rho[rho > 0])
-    elif ieos == 2:
-        cs[rho > 0] = np.sqrt(gamma * P[rho > 0] / rho[rho > 0])
-    else:
-        raise ValueError('Cannot determine equation of state')
-    return cs
+        return np.sqrt(P / rho)
+    if ieos == 2:
+        return np.sqrt(gamma * P / rho)
+    raise ValueError('Unknown equation of state')
 
 
 def _stopping_time(snap: Snap) -> ndarray:
-    stopping_time = snap._file_pointer['particles/tstop'][()]
+    stopping_time = _get_dataset('tstop', 'particles')(snap)
     stopping_time[stopping_time == _bignumber] = np.inf
     return stopping_time
 
@@ -290,8 +332,7 @@ def _stopping_time(snap: Snap) -> ndarray:
 def _dust_fraction(snap: Snap) -> ndarray:
     if snap.properties['dust_method'] != 'dust/gas mixture':
         raise ValueError('Dust fraction only available for "dust/gas mixture"')
-    dust_fraction = snap._file_pointer['particles/dustfrac'][()]
-    return dust_fraction
+    return _get_dataset('dustfrac', 'particles')(snap)
 
 
 def _dust_to_gas_ratio(snap: Snap) -> ndarray:
@@ -299,5 +340,4 @@ def _dust_to_gas_ratio(snap: Snap) -> ndarray:
         raise ValueError(
             'Dust fraction only available for "dust as separate sets of particles"'
         )
-    dust_to_gas_ratio = snap._file_pointer['particles/dustfrac'][()]
-    return dust_to_gas_ratio
+    return _get_dataset('dustfrac', 'particles')(snap)
