@@ -1,15 +1,16 @@
 """The utils sub-package."""
 
+from . import sph
 from .geometry import cartesian_to_polar
 from .math import average, cross, norm
-from .utils import get_extent_from_percentile, is_documented_by, time_string
+from .utils import is_documented_by, time_string
 
 __all__ = [
     'average',
     'cartesian_to_polar',
     'cross',
     'is_documented_by',
-    'get_extent_from_percentile',
     'norm',
+    'sph',
     'time_string',
 ]
