@@ -5,14 +5,15 @@ simulations using kernel density estimation based interpolation.
 """
 
 from .animation import animation, animation_profiles
-from .functions import particle_plot, plot, plot_snaps, str_to_units
+from .functions import get_extent_from_percentile, str_to_units
 from .interpolation import interpolate
-from .visualization import Visualization
+from .multi import plot_snaps
+from .visualization import particle_plot, plot
 
 __all__ = [
-    'Visualization',
     'animation',
     'animation_profiles',
+    'get_extent_from_percentile',
     'interpolate',
     'particle_plot',
     'plot',
