@@ -7,10 +7,11 @@ import matplotlib as mpl
 import numpy as np
 from numpy import ndarray
 
+from .. import logger
 from .interpolation import Extent
 
 
-def particle_plot(
+def plot(
     *, x: ndarray, y: ndarray, ax: Any, **kwargs,
 ):
     """Plot particles.
@@ -33,11 +34,10 @@ def particle_plot(
     """
     _kwargs = copy(kwargs)
     fmt = _kwargs.pop('fmt', 'k.')
-    lines = ax.plot(x, y, fmt, **_kwargs)
-    return lines
+    return ax.plot(x, y, fmt, **_kwargs)
 
 
-def scatter_plot(
+def scatter(
     *,
     x: ndarray,
     y: ndarray,
@@ -77,7 +77,7 @@ def scatter_plot(
     if color is None and size is None:
         raise ValueError('Should set size or color')
     if n_samples > 100_000:
-        print('n_samples > 100,000: this may be slow')
+        logger.warning('n_samples > 100,000: this may be slow')
     if random_seed is not None:
         np.random.seed(random_seed)
 
@@ -93,14 +93,13 @@ def scatter_plot(
     _kwargs = copy(kwargs)
     alpha = _kwargs.pop('alpha', 0.5)
 
-    paths = ax.scatter(x, y, c=color, s=size, alpha=alpha, **_kwargs)
-    return paths
+    return ax.scatter(x, y, c=color, s=size, alpha=alpha, **_kwargs)
 
 
-def render_plot(
+def imshow(
     *, interpolated_data: ndarray, extent: Extent, ax: Any, **kwargs,
 ):
-    """Plot 1d interpolated data as a rendered image.
+    """Plot 1d interpolated data as an image.
 
     Parameters
     ----------
@@ -130,14 +129,12 @@ def render_plot(
     else:
         raise ValueError('Cannot determine normalization for colorbar')
 
-    image = ax.imshow(
+    return ax.imshow(
         interpolated_data, origin='lower', extent=extent, norm=norm, **_kwargs
     )
 
-    return image
 
-
-def contour_plot(
+def contour(
     *, interpolated_data: ndarray, extent: Extent, ax: Any, **kwargs,
 ):
     """Plot 1d interpolated data as a contour plot.
@@ -163,12 +160,10 @@ def contour_plot(
         np.linspace(*extent[:2], n_interp_x), np.linspace(*extent[2:], n_interp_y),
     )
 
-    contour = ax.contour(X, Y, interpolated_data, **kwargs)
-
-    return contour
+    return ax.contour(X, Y, interpolated_data, **kwargs)
 
 
-def quiver_plot(
+def quiver(
     *, interpolated_data: ndarray, extent: Extent, ax: Any, **kwargs,
 ):
     """Plot 2d interpolated data as a quiver plot.
@@ -211,12 +206,10 @@ def quiver_plot(
         U /= norm
         V /= norm
 
-    quiver = ax.quiver(X, Y, U, V, **_kwargs)
-
-    return quiver
+    return ax.quiver(X, Y, U, V, **_kwargs)
 
 
-def stream_plot(
+def streamplot(
     *, interpolated_data: ndarray, extent: Extent, ax: Any, **kwargs,
 ):
     """Plot 2d interpolated data as a stream plot.
@@ -243,6 +236,4 @@ def stream_plot(
     )
     U, V = interpolated_data[0], interpolated_data[1]
 
-    streamplot = ax.streamplot(X, Y, U, V, **kwargs)
-
-    return streamplot
+    return ax.streamplot(X, Y, U, V, **kwargs)
